@@ -1,13 +1,15 @@
+from .utils import catch_keyboard_interrupt
+
+
+@catch_keyboard_interrupt()
 def cli():
     from argparse import ArgumentParser
     from os import chdir, makedirs, remove, mkdir, getcwd
     from os.path import realpath, join, exists, splitext, basename
-    from sys import executable
     from shutil import rmtree, copyfileobj
     import pkg_resources
     from subprocess import run
     from pathlib import Path
-    import importlib.util
     import json
     import requests
 
@@ -48,13 +50,17 @@ def cli():
             return
         chdir(dest)
 
-    def write_file(path, obj, ext):
-        out = realpath(splitext(path)[0] + ext)
-        if exists(out):
-            remove(out)
+    def abs_path(path, ext):
+        return realpath(splitext(path)[0] + ext)
+
+    def write_file(path, obj, ext, script_block):
+        out = abs_path(path, ext)
         if not NO_CACHE:
             CACHED_OBJECTS[out] = obj
-        obj.write(out)
+        if script_block not in SKIP_WRITE:
+            if exists(out):
+                remove(out)
+            obj.write(out)
 
     def clean_path(file):
         path = realpath(file)
@@ -144,14 +150,6 @@ def cli():
         with open(out_path + '.json', 'w') as f:
             json.dump(script, f, indent=4)
 
-    # ------------------------------------- #
-    # CHECK PACKAGE INSTALLATION
-    # ------------------------------------- #
-
-    if importlib.util.find_spec('gamut') == None:
-        print_error(
-            "You haven't installed the GAMuT package. You can find the installation guide here: https://felipe-tovar-henao.com/gamut/installation")
-
     # ------------------------------------- #
     # DEFINE FOLDER STRUCTURE
     # ------------------------------------- #
@@ -163,41 +161,85 @@ def cli():
     CACHED_OBJECTS = {}
     SCRIPT_MODES = ['corpus', 'mosaic', 'audio']
 
-    parser = ArgumentParser(
-        prog='GAMuT parser', description="Command-line utility for creating GAMuT audio musaicings with JSON files",
-        epilog='To learn more, visit https://felipe-tovar-henao.com/gamut')
-    parser.add_argument('-v', '--version', action='store_true', help='get current version of GAMuT')
-    parser.add_argument('--test', action='store_true', help='runs GAMuT package test.')
-    parser.add_argument('--no-check', action='store_true', help='Disable directory checking when reading a script')
-    parser.add_argument('--no-cache', action='store_true', help='Disable pipeline caching')
-    parser.add_argument('-i', '--init', nargs='?', const=ROOT_DIR,
-                        help='initializes a project folder at the specified directory', type=str)
-    parser.add_argument('-s', '--script', help="set JSON file to use as input settings for GAMuT", type=str)
-    parser.add_argument('--summarize', help="show summary of a .gamut file", type=str)
-    parser.add_argument('-p', '--play', action='store_true', help="enable audio playback after script runs")
-    parser.add_argument('--skip', nargs='+', help="skip one or more parts of the script", choices=SCRIPT_MODES)
-    parser.add_argument('-t', '--template', nargs='?', const=TEST_SCRIPT_DIR, help="generate a JSON script template", type=str)
-
-    parser.add_argument('--source', nargs='+', help="path to corpus source(s)")
-    parser.add_argument('--params', nargs='+', help="audio control parameters")
-    parser.add_argument('--target', help="path to audio target", type=str)
-    parser.add_argument('--audio', default="gamut-output.wav", help='path to audio output', type=str)
+    parser = ArgumentParser(prog='GAMuT parser',
+                            description="Command-line utility for creating GAMuT audio musaicings with JSON files",
+                            epilog='To learn more, visit https://felipe-tovar-henao.com/gamut')
+    parser.add_argument('-v', '--version',
+                        action='store_true',
+                        help='get current version of GAMuT')
+    parser.add_argument('--test',
+                        action='store_true',
+                        help='runs GAMuT package test.')
+    parser.add_argument('--no-check',
+                        action='store_true',
+                        help='Disable directory checking when reading a script')
+    parser.add_argument('--no-cache',
+                        action='store_true',
+                        help='Disable pipeline caching')
+    parser.add_argument('-i', '--init',
+                        nargs='?',
+                        const=ROOT_DIR,
+                        help='initializes a project folder at the specified directory',
+                        type=str)
+    parser.add_argument('-s', '--script',
+                        help="set JSON file to use as input settings for GAMuT",
+                        type=str)
+    parser.add_argument('--summarize',
+                        help="show summary of a .gamut file",
+                        type=str)
+    parser.add_argument('-p', '--play',
+                        action='store_true',
+                        help="enable audio playback after script runs")
+    parser.add_argument('--skip',
+                        nargs='+',
+                        help="skip one or more parts of the script",
+                        choices=SCRIPT_MODES)
+    parser.add_argument('--skip-write',
+                        nargs='+',
+                        help="skip writing to disk one or more parts of the script",
+                        choices=SCRIPT_MODES)
+    parser.add_argument('-t', '--template',
+                        nargs='?',
+                        const=TEST_SCRIPT_DIR,
+                        help="generate a JSON script template",
+                        type=str)
+    parser.add_argument('--source',
+                        nargs='+',
+                        help="path to corpus source(s)")
+    parser.add_argument('--params',
+                        nargs='+',
+                        help="audio control parameters")
+    parser.add_argument('--features',
+                        nargs='+',
+                        help="audio features to base audio musaicing on",
+                        choices=['pitch', 'analysis'])
+    parser.add_argument('--target',
+                        help="path to audio target",
+                        type=str)
+    parser.add_argument('--audio',
+                        default="gamut-output.wav",
+                        help='path to audio output',
+                        type=str)
 
     args = parser.parse_args()
 
     NO_CACHE = args.no_cache
+    SKIP_WRITE = args.skip_write or []
+    raw_musaicing_args = [args.source, args.target]
+
+    if len(SKIP_WRITE) > 0 and NO_CACHE:
+        print_error("You can't use --skip-write and --no-cache at the same time.")
 
     # ------------------------------------- #
     # PRINT GAMUT PACKAGE VERSION
     # ------------------------------------- #
     if args.version:
         print_success(f'GAMuT v{pkg_resources.get_distribution("gamut").version}')
-        exit()
 
     # ------------------------------------- #
     # RUN PACKAGE TEST
     # ------------------------------------- #
-    if args.test:
+    elif args.test:
         TEST_DIR = '.gamut-test'
         mkdir(TEST_DIR)
         TEST_DIR = realpath(TEST_DIR)
@@ -209,34 +251,36 @@ def cli():
         except Exception as e:
             print(e)
         rmtree(TEST_DIR)
-        exit()
 
     # ------------------------------------- #
     # PROCESS RAW INPUT
     # ------------------------------------- #
-    raw_musaicing_args = [args.source, args.target]
-    if any(raw_musaicing_args):
-        params = parse_params(args.params)
+    elif any(raw_musaicing_args):
         if not all(raw_musaicing_args):
             print_error("You must provide both source and target")
+        params = parse_params(args.params)
         source, target = raw_musaicing_args
-        sources = [clean_path(s) for s in source]
         target = clean_path(target)
+        corpus_params = {
+            'source': [clean_path(s) for s in source]
+        }
+        if args.features:
+            corpus_params['features'] = args.features
 
-        from gamut.features import Corpus, Mosaic
+        from .features import Corpus, Mosaic
 
-        corpus = Corpus(source=sources)
+        corpus = Corpus(**corpus_params)
         mosaic = Mosaic(target=target, corpus=corpus)
         audio = mosaic.to_audio(**params)
-        write_file(args.audio, audio, ".wav")
+        if 'audio' not in SKIP_WRITE:
+            write_file(args.audio, audio, ".wav", 'audio')
         if args.play:
             audio.play()
-        exit()
 
     # ------------------------------------- #
     # INITIALIZE WORKSPACE
     # ------------------------------------- #
-    if args.init:
+    elif args.init:
         if realpath(args.init) != ROOT_DIR:
             define_directories(args.init)
         for x in [MOSAIC_DIR, CORPUS_DIR, AUDIO_DIR, SCRIPTS_DIR]:
@@ -260,7 +304,6 @@ def cli():
 
         print_success(
             f"Your GAMuT project folder is ready!\nTry running:\n\tgamut --script scripts/{TEST_NAME}.json --play")
-        exit()
 
     # ------------------------------------- #
     # CREATE TEMPLATE
@@ -269,7 +312,6 @@ def cli():
     elif args.template:
         safe_chdir(SCRIPTS_DIR)
         create_new_template(args.template)
-        exit()
 
     # ------------------------------------- #
     # SUMMARIZE GAMUT FILE
@@ -283,13 +325,14 @@ def cli():
         if splitext(gamut_file)[1] != '.gamut':
             print_error(f'{gamut_file} is not a .gamut file')
 
-        from gamut.features import Corpus, Mosaic
+        from .features import Corpus, Mosaic
+
         for obj in [Corpus(), Mosaic()]:
             try:
                 obj.read(gamut_file).summarize()
             except:
                 continue
-            exit()
+            break
 
     # ------------------------------------- #
     # PROCESS SCRIPT
@@ -304,93 +347,93 @@ def cli():
         with open(file=script_path, mode='r',) as f:
             script = json.loads(f.read())
 
-        for mode in script:
-            if mode not in SCRIPT_MODES:
-                print_error(f'"{mode}" is not a valid GAMuT script key')
+        for script_block in script:
+            if script_block not in SCRIPT_MODES:
+                print_error(f'"{script_block}" is not a valid GAMuT script key')
 
-        from gamut.features import Corpus, Mosaic
+        from .features import Corpus, Mosaic
 
         skip = args.skip if args.skip else []
         sorted_modes = [m for m in SCRIPT_MODES if m in script and m not in skip]
 
-        for mode in sorted_modes:
-            params = script[mode]
+        for script_block in sorted_modes:
+            params = script[script_block]
             name = params.pop('name', None)
-            output_name = name if name else splitext(basename(script_path))[0] + f'-{mode}'
+            output_name = name if name else splitext(basename(script_path))[0] + f'-{script_block}'
 
-            match mode:
-                case 'corpus':
-                    safe_chdir(AUDIO_DIR)
+            if script_block == 'corpus':
+                safe_chdir(AUDIO_DIR)
 
-                    for i, source in enumerate(params['source']):
-                        params['source'][i] = clean_path(source)
-                    safe_chdir(ROOT_DIR)
-                    c = Corpus(**params)
-                    safe_chdir(CORPUS_DIR)
-                    write_file(output_name, c, '.gamut')
+                for i, source in enumerate(params['source']):
+                    params['source'][i] = clean_path(source)
+                safe_chdir(ROOT_DIR)
+                c = Corpus(**params)
+                safe_chdir(CORPUS_DIR)
+                write_file(output_name, c, '.gamut', script_block)
 
-                case 'mosaic':
-                    for x in ['corpus', 'target']:
-                        if x not in params:
-                            print_error(f'You forgot to specify a {x} in your {mode} script.')
+            elif script_block == 'mosaic':
+                for x in ['corpus', 'target']:
+                    if x not in params:
+                        print_error(f'You forgot to specify a {x} in your {script_block} script.')
 
-                    # clean target path
-                    safe_chdir(AUDIO_DIR)
-                    params['target'] = clean_path(params['target'])
+                # clean target path
+                safe_chdir(AUDIO_DIR)
+                params['target'] = clean_path(params['target'])
 
-                    # clean beat unit
-                    if 'beat_unit' in params:
-                        try:
-                            if "/" in params['beat_unit']:
-                                a, b = params['beat_unit'].split('/')
-                                params['beat_unit'] = int(a) / int(b)
-                            else:
-                                params['beat_unit'] = int(params['beat_unit'])
-                        except:
-                            print_error('Invalid "beat_unit" value')
-
-                    # clean corpus paths
-                    safe_chdir(CORPUS_DIR)
-                    corpus_paths = params.pop('corpus')
-                    corpora = []
-                    for c in corpus_paths:
-                        cpath = clean_path(splitext(c)[0] + ".gamut")
-                        if cpath in CACHED_OBJECTS:
-                            corpora.append(CACHED_OBJECTS[cpath])
+                # clean beat unit
+                if 'beat_unit' in params:
+                    try:
+                        if "/" in params['beat_unit']:
+                            a, b = params['beat_unit'].split('/')
+                            params['beat_unit'] = int(a) / int(b)
                         else:
-                            corpora.append(Corpus().read(cpath))
+                            params['beat_unit'] = int(params['beat_unit'])
+                    except:
+                        print_error('Invalid "beat_unit" value')
 
-                    # build mosaic
-                    safe_chdir(ROOT_DIR)
-                    m = Mosaic(corpus=corpora, **params)
+                # clean corpus paths
+                safe_chdir(CORPUS_DIR)
+                corpus_paths = params.pop('corpus')
+                corpora = []
+                for c in corpus_paths:
+                    cpath = abs_path(c, '.gamut')
+                    if cpath in CACHED_OBJECTS:
+                        corpora.append(CACHED_OBJECTS[cpath])
+                        continue
+                    cpath = clean_path(cpath)
+                    corpora.append(Corpus().read(cpath))
 
-                    # write to disk
-                    safe_chdir(MOSAIC_DIR)
-                    write_file(output_name, m, '.gamut')
+                # build mosaic
+                safe_chdir(ROOT_DIR)
+                m = Mosaic(corpus=corpora, **params)
 
-                case 'audio':
-                    mosaic_file = params.pop('mosaic')
+                # write to disk
+                safe_chdir(MOSAIC_DIR)
+                write_file(output_name, m, '.gamut', script_block)
 
-                    # clean mosaic path
-                    safe_chdir(MOSAIC_DIR)
-                    mosaic_path = clean_path(splitext(mosaic_file)[0] + ".gamut")
-                    m = Mosaic().read(mosaic_path) if mosaic_path not in CACHED_OBJECTS else CACHED_OBJECTS[mosaic_path]
+            elif script_block == 'audio':
+                mosaic_file = params.pop('mosaic')
 
-                    # create audio mosaic
-                    safe_chdir(ROOT_DIR)
-                    audio = m.to_audio(**params)
-                    safe_chdir(AUDIO_DIR)
+                # clean mosaic path
+                safe_chdir(MOSAIC_DIR)
+                mosaic_path = abs_path(mosaic_file, '.gamut')
+                m = Mosaic().read(clean_path(mosaic_path)
+                                  ) if mosaic_path not in CACHED_OBJECTS else CACHED_OBJECTS[mosaic_path]
 
-                    # clean convolution
-                    convolve = params.pop('convolve', None)
-                    if convolve:
-                        if 'impulse_response' not in convolve:
-                            print_error('To apply audio convolution, you must provide an inpulse response')
-                        convolve['impulse_response'] = clean_path(convolve['impulse_response'])
-                        audio.convolve(**convolve)
-                    write_file(output_name, audio, '.wav')
-                    if args.play:
-                        audio.play()
-        exit()
+                # create audio mosaic
+                safe_chdir(ROOT_DIR)
+                audio = m.to_audio(**params)
+                safe_chdir(AUDIO_DIR)
+
+                # clean convolution
+                convolve = params.pop('convolve', None)
+                if convolve:
+                    if 'impulse_response' not in convolve:
+                        print_error('To apply audio convolution, you must provide an inpulse response')
+                    convolve['impulse_response'] = clean_path(convolve['impulse_response'])
+                    audio.convolve(**convolve)
+                write_file(output_name, audio, '.wav', script_block)
+                if args.play:
+                    audio.play()
     else:
-        print_error("You didn't provide any arguments. Use -h or --help to learn more.")
+        print_error("You didn't provide any required arguments. Use -h or --help to learn more.")
